@@ -24,14 +24,11 @@
 
 #include "system.h"
 #include "cycle-check.h"
-#include "dirfd.h"
 #include "error.h"
-#include "euidaccess.h"
 #include "euidaccess-stat.h"
 #include "file-type.h"
 #include "hash.h"
 #include "hash-pjw.h"
-#include "lstat.h"
 #include "obstack.h"
 #include "quote.h"
 #include "remove.h"
@@ -1276,7 +1273,8 @@ dirent_count (struct stat const *st)
   return st->st_size / 16;
 }
 
-# if HAVE_SYS_VFS_H && HAVE_FSTATFS && HAVE_STRUCT_STATFS_F_TYPE
+#if defined __linux__ \
+  && HAVE_SYS_VFS_H && HAVE_FSTATFS && HAVE_STRUCT_STATFS_F_TYPE
 #  include <sys/vfs.h>
 #  include "fs.h"
 
@@ -1422,7 +1420,7 @@ preprocess_dir (DIR **dirp, struct rm_options const *x)
   for (size_t i = 0; i < n; i++)
     {
       /* ignore failure */
-      (void) unlinkat (dir_fd, vv[i]->name, 0);
+      unlinkat (dir_fd, vv[i]->name, 0);
     }
 
  cleanup:
