@@ -118,19 +118,16 @@
 /* The user to become if none is specified.  */
 #define DEFAULT_USER "root"
 
-char *crypt ();
-char *getusershell ();
-void endusershell ();
-void setusershell ();
+char *crypt (char const *key, char const *salt);
+char *getusershell (void);
+void endusershell (void);
+void setusershell (void);
 
 extern char **environ;
 
 static void run_shell (char const *, char const *, char **, size_t)
      ATTRIBUTE_NORETURN;
 
-/* The name this program was run with.  */
-char *program_name;
-
 /* If true, pass the `-f' option to the subshell.  */
 static bool fast_startup;
 
@@ -414,7 +411,7 @@ main (int argc, char **argv)
   struct passwd pw_copy;
 
   initialize_main (&argc, &argv);
-  program_name = argv[0];
+  set_program_name (argv[0]);
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
