@@ -34,7 +34,6 @@
 #include "canon-host.h"
 #include "readutmp.h"
 #include "error.h"
-#include "hard-locale.h"
 #include "quote.h"
 
 /* The official name of this program (e.g., no `g' prefix).  */
@@ -96,7 +95,7 @@
 # define UT_ID(U) "??"
 #endif
 
-char *ttyname ();
+char *ttyname (int);
 
 /* If true, attempt to canonicalize hostnames via a DNS lookup. */
 static bool do_lookup;
